@@ -1,4 +1,4 @@
-# This file is part of cclib (http://cclib.sf.net), a library for parsing
+# This file is part of cclib (http://cclib.github.io), a library for parsing
 # and interpreting the results of computational chemistry packages.
 #
 # Copyright (C) 2006, the cclib development team
@@ -8,7 +8,7 @@
 # received a copy of the license along with cclib. You can also access
 # the full license online at http://www.gnu.org/copyleft/lgpl.html.
 
-__revision__ = "$Revision: 960 $"
+"""Bridge for using cclib data in biopython (http://biopython.org)."""
 
 from Bio.PDB.Atom import Atom
 from cclib.parser.utils import PeriodicTable
