@@ -1,16 +1,23 @@
-# This file is part of cclib (http://cclib.sf.net), a library for parsing
+# This file is part of cclib (http://cclib.github.io), a library for parsing
 # and interpreting the results of computational chemistry packages.
 #
-# Copyright (C) 2006, the cclib development team
+# Copyright (C) 2006-2013, the cclib development team
 #
 # The library is free software, distributed under the terms of
 # the GNU Lesser General Public version 2.1 or later. You should have
 # received a copy of the license along with cclib. You can also access
 # the full license online at http://www.gnu.org/copyleft/lgpl.html.
 
-__revision__ = "$Revision: 960 $"
+"""Bridge for using cclib data in PyQuante (http://pyquante.sourceforge.net)."""
 
-from PyQuante.Molecule import Molecule
+from __future__ import print_function
+
+import sys
+
+try:
+    from PyQuante.Molecule import Molecule
+except ImportError:
+    print("PyQuante could not be imported.")
 
 
 def makepyquante(atomcoords, atomnos, charge=0, mult=1):
@@ -25,7 +32,7 @@ def makepyquante(atomcoords, atomnos, charge=0, mult=1):
     >>> print int(en * 10) / 10. # Should be around -73.8
     -73.8
     """
-    return Molecule("notitle", zip(atomnos, atomcoords), units="Angstrom",
+    return Molecule("notitle", list(zip(atomnos, atomcoords)), units="Angstrom",
                     charge=charge, multiplicity=mult)
 
 
